Imports System.Collections.Specialized

Public Class PartialOrdering
    ' **************************************************
    Public Class NoPartialOrderingException
        Inherits Exception

        Public Sub New()
            MyBase.New("Data cannot be partially ordered")
        End Sub
    End Class
    ' **************************************************
    ' Represents an item in a partial ordering.
    Private Class PoItem
        Public Name As String
        Public NumBeforeMe As Long
        Public ItemsAfterMe As New Collection

        Public Sub New(ByVal new_name As String)
            Name = new_name
        End Sub
    End Class
    ' **************************************************

    ' An item's entry holds the items that follow it.
    Private m_Items As New Collection

    ' Add a relationship between these two items.
    Public Sub AddRelation(ByVal comes_before_name As String, ByVal comes_after_name As String)
        ' See if comes_before_name is present.
        Dim comes_before_object As PoItem
        Try
            comes_before_object = m_Items(comes_before_name)
        Catch ex As Exception
            comes_before_object = New PoItem(comes_before_name)
            m_Items.Add(comes_before_object, comes_before_name)
        End Try

        ' See if comes_after_name is present.
        Dim comes_after_object As PoItem
        Try
            comes_after_object = m_Items(comes_after_name)
        Catch ex As Exception
            comes_after_object = New PoItem(comes_after_name)
            m_Items.Add(comes_after_object, comes_after_name)
        End Try

        ' Record the relationship.
        comes_before_object.ItemsAfterMe.Add(comes_after_object)
        comes_after_object.NumBeforeMe += 1
    End Sub

    ' Return an array listing the items in a valid ordering.
    Public Function ExtendPartialOrdering() As String()
        ' Find the items that have no prerequisites.
        Dim no_prerequisites As New Collection
        For Each po_item As PoItem In m_Items
            If po_item.NumBeforeMe = 0 Then
                no_prerequisites.Add(po_item)
                m_Items.Remove(po_item.Name)
            End If
        Next po_item

        ' Output items with no prerequisites.
        Dim output_item As PoItem
        Dim results() As String
        Dim num_results As Integer
        Do While no_prerequisites.Count > 0
            ' Remove the first item with no prerequisites.
            output_item = no_prerequisites(1)
            no_prerequisites.Remove(1)

            ' Add the item to the output.
            num_results += 1
            ReDim Preserve results(num_results - 1)
            results(num_results - 1) = output_item.Name

            ' Decrement NumAfterMe for items that follow
            ' this one.
            For Each after_me As PoItem In output_item.ItemsAfterMe
                after_me.NumBeforeMe -= 1
                ' See if there are no more prerequisites.
                If after_me.NumBeforeMe = 0 Then
                    ' Add to no_prerequisites.
                    no_prerequisites.Add(after_me)
                    m_Items.Remove(after_me.Name)
                End If
            Next after_me
        Loop

        ' See if there are any items still in m_Items.
        If m_Items.Count > 0 Then Throw New NoPartialOrderingException

        ' Return the results.
        Return results
    End Function
End Class
