Public Class EmployeeCollection
    Inherits System.Collections.CollectionBase

    ' Add an Employee object.
    Public Sub Add(ByVal new_employee As Employee)
        ' See if the employee is already present.
        If Not (Item(new_employee.LastName, new_employee.FirstName) Is Nothing) Then
            Throw New System.ArgumentException("Duplicate Employee")
        End If

        ' Add the object to the List 
        ' provided by CollectionBase.
        List.Add(new_employee)
    End Sub

    ' Add an Employee object using its name values.
    Public Function Add(ByVal last_name As String, ByVal first_name As String) As Employee
        ' See if the employee is already present.
        If Not (Item(last_name, first_name) Is Nothing) Then
            Throw New System.ArgumentException("Duplicate Employee")
        End If

        ' Add the object to the List 
        ' provided by CollectionBase.
        Dim new_employee As New Employee(last_name, first_name)
        List.Add(new_employee)

        Return new_employee
    End Function

    ' Remove an Employee object by index.
    Public Sub Remove(ByVal index As Integer)
        ' See if the index exists.
        If (index < 0) Or (index > Count - 1) Then
            Throw New System.IndexOutOfRangeException("Invalid Employee index")
        Else
            ' Remove the item from List.
            List.RemoveAt(index)
        End If
    End Sub

    ' Remove an Employee by name.
    Public Sub Remove(ByVal last_name As String, ByVal first_name As String)
        ' See if the Employee exists.
        Dim target As Employee = Item(last_name, first_name)
        If target Is Nothing Then
            ' There's no such employee.
            Throw New System.IndexOutOfRangeException("Employee not found")
        Else
            ' Remove the employee.
            List.Remove(target)
        End If
    End Sub

    ' Return an Employee by index.
    Public ReadOnly Property Item(ByVal index As Integer) As Employee
        Get
            ' See if the index exists.
            If (index < 0) Or (index > Count - 1) Then
                Return Nothing
            Else
                ' Return the Employee.
                Return CType(List.Item(index), Employee)
            End If
        End Get
    End Property

    ' Return an Employee by name.
    Public ReadOnly Property Item(ByVal last_name As String, ByVal first_name As String) As Employee
        Get
            ' See if the Employee exists.
            Dim test_employee As Employee
            For i As Long = 0 To list.Count - 1
                test_employee = list(i)
                If test_employee.LastName = last_name And _
                   test_employee.FirstName = first_name _
                Then
                    Return CType(List.Item(i), Employee)
                End If
            Next i

            ' The Employee isn't here.
            Return Nothing
        End Get
    End Property
End Class
